/***************************************************************************
 *   aio_admin.sma            Version 1.0 Release          Date: MAR/08/2004
 *
 *   RS All-In-One[AIO] Admin Menu (Small Source File)
 *
 *   By:    Rob Secord
 *   Alias: xeroblood (aka; sufferer)                     xeroblood@msn.com
 *          -dd| Digital Darkness        http://www.digitaldarkness.vze.com
 *
 * *******************************************************************************
 *   
 *	Ported By KingPin( kingpin@onexfx.com ). I take no responsibility 
 *	for this file in any way. Use at your own risk. No warranties of any kind. 
 *
 * *******************************************************************************
 *
 *   Developed using:  AMXX 0.16
 *   Tested On:        CS 1.6 (STEAM)
 * 
 * *******************************************************************************
 * 
 * Changelog : 
 *
 * .3	Fixed the help file paths. 
 *	changed the last timer option from 20 minutes to 30 for a standard map time.
 *	changed [AMX] to [AMXX].
 *	added a 0 second option to turn off commands like fire/turbo etc.
 *	Removed the 800 (15 minute) option from above commands.
 *    	fixed a coupla typos.
 *    
 * 
 * .2c ported to amxx changed some minor things by KingPin.
 *
 * .2c and above by KingPin.
 *
 * .2b maintained by xeroblood.
 *
 **************************************************************************/

#include <amxmodx>
#include <amxmisc>
#include <fun>          //  Used for set_user_footsteps()
#include <engine>
#include <cstrike>
#include <aio_admin>        // Custom include file (Defines, Functions)

new const AIO_TITLE[] = "RS-AIO Admin Menu"
new const AIO_AUTHOR[] = "xeroblood"
new const AIO_VERSION[] = "0.3"


/***************************************************************************
 ********  Variables Section  **********************************************
 **************************************************************************/

new g_szMode[2][] = { "DISABLED", "ENABLED" }
new g_szColors[MAX_COLORS][] = { "Red", "Green", "Blue", "Cyan", "Magenta", "Yellow", "Orange", "Random" }
new g_nColors[MAX_COLORS][] =
{
    {255, 0, 0},    // Red
    {0, 255, 0},    // Green
    {0, 0, 255},    // Blue
    {0, 255, 255},  // Cyan
    {255, 0, 255},  // Magenta
    {255, 255, 0},  // Yellow
    {255, 153, 0},  // Orange
    {0, 0, 0}       // Not Used (Random)
}

// CS Player Models
new g_szCTSkins[4][] = { "sas", "gsg9", "urban", "gign" }
new g_szTSkins[4][] =  { "arctic", "leet", "guerilla", "terror" }

// Gravity Setting (1.0 = 800 = Normal Gravity)
new Float:g_fGravity[] = { 1.0, 0.25 }

// Current Player Command/Effect
new g_szCmdArgs[MAX_PLAYERS][STR_T]
new g_nHasEffect[MAX_PLAYERS]
new g_nReviveTimes[MAX_PLAYERS]

// Client Messages (Engine)
new g_nMsgDamage
new g_nMsgDeath

// Sprites
new g_sprWhite
new g_sprSmoke
new g_sprMFlash
new g_sprBFlare

// Specific Command Variables
new g_nRocket_Z[MAX_PLAYERS]
new Float:g_fSpeeds[MAX_PLAYERS]

// Menu Variables
new g_nMenuPlayers[MAX_PLAYERS][MAX_PLAYERS]
new g_nMenuPlayersNum[MAX_PLAYERS]
new g_nMenuPosition[MAX_PLAYERS]
new g_nMenuOrder[MAX_PLAYERS]
new g_nCurrPlayer[MAX_PLAYERS]
new g_nCurrCommand[MAX_PLAYERS]
new g_nCurrType[MAX_PLAYERS]

// HTML Files For Help/Warn MOTDs
new g_szFileWarn[] = "/addons/amxx/custom/aio_html/aio_warning.htm"
new g_szFileHelp[] = "/addons/amxx/custom/aio_html/aio_help.htm"

// Sound Files
new g_szSndFlames[]  = "ambience/flameburst1.wav"
new g_szSndScream1[] = "scientist/scream07.wav"
new g_szSndScream2[] = "scientist/scream21.wav"
new g_szSndRocket1[] = "weapons/rocketfire1.wav"
new g_szSndRocket2[] = "weapons/rocket1.wav"
new g_szSndPoison[]  = "player/bhit_kevlar-1.wav"

// Command Arrays
new g_szCmdTitle[MAX_COMMANDS][] =
{
    "Ban",       "Kick",       "Slap",       "Bitch Slap",  "Bury",        "Gag",         "Mute",
    "Warn",      "Blanks",     "Fire",       "Rocket",      "Strip Guns",  "Poison",
    "God Mode",  "Low Gravity","No-Clip",    "Turbo",       "Silent Mode", "Invisibility","Chameleon",
    "Free Nades","Free Ammo",  "Health",     "Armor",       "Revival",     "Big-Ears",
    "Stack",     "Glow",       "Cash-Editor","Admin-Editor","Frags-Editor","Weapon-Editor"
}
new g_szAdminCVAR[MAX_COMMANDS][] =
{
    "sv_aio_ban",       "_",       "sv_aio_slap",  "sv_aio_bslap",      "_",       "sv_aio_gag",  "sv_aio_mute",
         "_",      "sv_aio_blanks","sv_aio_fire",       "_",       "sv_aio_strip", "sv_aio_poison",
    "sv_aio_god",  "sv_aio_grav",  "sv_aio_noclip","sv_aio_turbo", "sv_aio_silent","sv_aio_invis","sv_aio_spy",
    "sv_aio_nades","sv_aio_ammo",  "sv_aio_health","sv_aio_armor", "sv_aio_revive","sv_aio_ears",
         "_",           "_",       "sv_aio_cash",  "sv_aio_admin", "sv_aio_frags", "sv_aio_weapon"
}
new g_szAdminCmd[MAX_COMMANDS][] =
{
    "aio_ban",  "aio_kick",  "aio_slap",  "aio_bslap", "aio_bury",  "aio_gag",  "aio_mute",
    "aio_warn", "aio_blanks","aio_fire",  "aio_rocket","aio_strip", "aio_poison",
    "aio_god",  "aio_grav",  "aio_noclip","aio_turbo", "aio_silent","aio_invis","aio_spy",
    "aio_nades","aio_ammo",  "aio_health","aio_armor", "aio_revive","aio_ears",
    "aio_stack","aio_glow",  "aio_cash",  "aio_admin", "aio_frags", "aio_weapon"
}
new g_szCmdFuntion[MAX_COMMANDS][] =
{
    "ActionBan",  "ActionKick",  "ActionSlap",  "ActionBitchSlap","ActionBury",   "ActionGag",  "ActionMute",
    "ActionWarn", "ActionBlanks","ActionFire",  "ActionRocket",   "ActionStrip",  "ActionPoison",
    "ActionGod",  "ActionGrav",  "ActionNoclip","ActionTurbo",    "ActionSilent", "ActionInvis","ActionSpy",
    "ActionNades","ActionAmmo",  "ActionHealth","ActionArmor",    "ActionRevival","ActionEars",
    "ActionStack","ActionGlow",  "ActionCash",  "ActionAdmin",    "ActionFrags",  "ActionWeapon"
}


/***************************************************************************
 ********  Plugin Forwards Section  ****************************************
 **************************************************************************/

public plugin_init()
{
    register_plugin( AIO_TITLE, AIO_VERSION, AIO_AUTHOR )

    register_clcmd( "aio_mode", "ClCmd_AIOMode", AIO_LEVEL, "<on|off> - Toggles AIO-Admin On/Off" )
    register_clcmd( "aio_help", "ClCmd_AIOHelp", AIO_LEVEL, "- Displays AIO-Admin Help Information (MOTD)" )
    register_clcmd( "aio_menu", "ClCmd_AIOMenu", AIO_LEVEL, "- Displays AIO-Admin Command Menu." )

    register_clcmd( "say",      "ClCmd_CheckSay" )
    register_clcmd( "say_team", "ClCmd_CheckSay" )

    for( new i = 0; i < MAX_COMMANDS; i++ )
        register_clcmd( g_szAdminCmd[i], "ClCmd_AdminCmd", AIO_LEVEL, "- Use aio_help Command for Usage." )

    // Registered Events
    register_event( "DeathMsg",  "Event_DeathMsg",  "a" )
    register_event( "ResetHUD",  "Event_NewRound",  "b" )
    register_event( "CurWeapon", "Event_ChgWeapon", "be", "1=1" )

    // Registered Menus
    register_menucmd( register_menuid("\yAIO Player Menu"),   1023, "Menu_AIONext" )
    register_menucmd( register_menuid("\yAIO Command Menu"),  1023, "Menu_AIONext" )
    register_menucmd( register_menuid("\yAIO Command Types"), 1023, "Menu_AIOType" )
    register_menucmd( register_menuid("\yAIO Settings Menu"), 1023, "Menu_AIOAction" )

    // Set Maximum Player Speed
    server_cmd( "sv_maxspeed 1000" )

    // Client Message Indexes
    g_nMsgDamage = get_user_msgid( "Damage" )
    g_nMsgDeath  = get_user_msgid( "DeathMsg" )

    // Plugin-Specific CVARs
    register_cvar( "sv_aio_allow",         "1" )
    register_cvar( "sv_aio_immunity_bad",  "1" )
    register_cvar( "sv_aio_immunity_good", "0" )
    register_cvar( "sv_aio_immunity_misc", "0" )
    register_cvar( "sv_aio_menu_start",    "1" )
    register_cvar( "sv_aio_command_flags", "abcdefghijklmnopqrstuvwxyz123456" )

    // Command-Specific CVARs
    register_cvar( "sv_aio_ban_ip",        "0" )    // 1=Ban IP, 0=Ban STEAM ID
    register_cvar( "sv_aio_fire_damage",   "5" )    // Fire Damage / Second
    register_cvar( "sv_aio_poison_damage", "5" )    // Poison Damage / Second
    register_cvar( "sv_aio_turbo_speed",   "1000" ) // Turbo Speed (Range 400 - 1000)

    // Menu-Specific CVARs [Max 9 Options / CVAR]
    register_cvar( "sv_aio_ban",    "0 15 30 60 90 120 300 600 1440" )
    register_cvar( "sv_aio_slap",   "1 5 10 15 20 25 50 75 99" )
    register_cvar( "sv_aio_bslap",  "10 20 30 40 50 60 70 80 90" )
    register_cvar( "sv_aio_gag",    "0 15 30 60 90 120 300 600 1800" )
    register_cvar( "sv_aio_mute",   "0 15 30 60 90 120 300 600 1800" )
    register_cvar( "sv_aio_blanks", "0 15 30 60 90 120 300 600 1800" )
    register_cvar( "sv_aio_fire",   "2 5 10 11 12 13 14 15 20" )
    register_cvar( "sv_aio_poison", "2 5 10 11 12 13 14 15 20" )
    register_cvar( "sv_aio_god",    "0 15 30 60 90 120 300 600 1800" )
    register_cvar( "sv_aio_grav",   "0 15 30 60 90 120 300 600 1800" )
    register_cvar( "sv_aio_noclip", "0 15 30 60 90 120 300 600 1800" )
    register_cvar( "sv_aio_turbo",  "0 15 30 60 90 120 300 600 1800" )
    register_cvar( "sv_aio_silent", "0 15 30 60 90 120 300 600 1800" )
    register_cvar( "sv_aio_invis",  "0 15 30 60 90 120 300 600 1800" )
    register_cvar( "sv_aio_spy",    "0 15 30 60 90 120 300 600 1800" )
    register_cvar( "sv_aio_nades",  "0 15 30 60 90 120 300 600 1800" )
    register_cvar( "sv_aio_ammo",   "0 15 30 60 90 120 300 600 1800" )
    register_cvar( "sv_aio_health", "25 50 75 100 125 150 175 200 255" )
    register_cvar( "sv_aio_armor",  "25 50 75 100 125 150 175 200 255" )
    register_cvar( "sv_aio_revive", "1 2 3 4 5 6 7 8 9" )
    register_cvar( "sv_aio_ears",   "0 15 30 60 90 120 300 600 1800" )
    register_cvar( "sv_aio_cash",   "0 800 1500 3000 6000 9000 12000 14000 16000" )
    register_cvar( "sv_aio_admin",  "ijm ijn ijo ijp ijq ijr ijs ijt abcde" )
    register_cvar( "sv_aio_frags",  "-3 -2 -1 1 2 3 4 5 6" )
    register_cvar( "sv_aio_weapon", "0 3 4 14 15 18 22 26 28" )
}

public plugin_precache()
{
    g_sprWhite  = precache_model( "sprites/white.spr" )
    g_sprSmoke  = precache_model( "sprites/steam1.spr" )
    g_sprMFlash = precache_model( "sprites/muzzleflash.spr" )
    g_sprBFlare = precache_model( "sprites/blueflare2.spr" )

    precache_sound( g_szSndFlames )
    precache_sound( g_szSndScream1 )
    precache_sound( g_szSndScream2 )
    precache_sound( g_szSndRocket1 )
    precache_sound( g_szSndRocket2 )
    precache_sound( g_szSndPoison )
}

public client_authorized( id )
{
    g_nHasEffect[id-1] = 0
    g_nReviveTimes[id-1] = 0
}

public client_disconnect( id )
{
    g_nHasEffect[id-1] = 0
    g_nReviveTimes[id-1] = 0
}

/***************************************************************************
 ********  Event Routines Section  *****************************************
 **************************************************************************/

public Event_DeathMsg()
{
    if( !get_cvar_num( "sv_aio_allow" ) )
        return PLUGIN_HANDLED

    new id = read_data(2)

    if( task_exists( TASK_BSLAP + id ) )  remove_task( TASK_BSLAP + id )
    if( task_exists( TASK_FIRE + id ) )   remove_task( TASK_FIRE + id )
    if( task_exists( TASK_POISON + id ) ) remove_task( TASK_POISON + id )

    if( g_nHasEffect[id-1] & AIO_BSLAP )  g_nHasEffect[id-1] ^= AIO_BSLAP
    if( g_nHasEffect[id-1] & AIO_FIRE )   g_nHasEffect[id-1] ^= AIO_FIRE
    if( g_nHasEffect[id-1] & AIO_POISON ) g_nHasEffect[id-1] ^= AIO_POISON

    if( g_nHasEffect[id-1] & AIO_REVIVE )
    {
        new aCmdArgs[2]
        aCmdArgs[0] = id

        if( g_nReviveTimes[id-1] > 0 )
        {
            g_nReviveTimes[id-1]--
            client_print( id, print_chat, "Revival Lives Left: %d", g_nReviveTimes[id-1] )
            set_task( 0.5, "RevivePlayer", (TASK_REVIVE + id), aCmdArgs, 1 )
        }else
            DeadPlayer( aCmdArgs )
    }else
    {
        // Unrevived players lose GOOD effects..
        if( task_exists( TASK_NADES + id ) )  remove_task( TASK_NADES + id )
        if( task_exists( TASK_AMMO + id ) )   remove_task( TASK_AMMO + id )

        if( g_nHasEffect[id-1] & AIO_NADES )  g_nHasEffect[id-1] ^= AIO_NADES
        if( g_nHasEffect[id-1] & AIO_AMMO )   g_nHasEffect[id-1] ^= AIO_AMMO
    }
    return PLUGIN_CONTINUE
}

public Event_NewRound( id )
{
    if( !get_cvar_num( "sv_aio_allow" ) )
        return PLUGIN_HANDLED

    if( task_exists( TASK_BSLAP + id ) )  remove_task( TASK_BSLAP + id )
    if( task_exists( TASK_FIRE + id ) )   remove_task( TASK_FIRE + id )
    if( task_exists( TASK_POISON + id ) ) remove_task( TASK_POISON + id )
    if( task_exists( TASK_NADES + id ) )  remove_task( TASK_NADES + id )

    if( g_nHasEffect[id-1] & AIO_BSLAP )  g_nHasEffect[id-1] ^= AIO_BSLAP
    if( g_nHasEffect[id-1] & AIO_FIRE )   g_nHasEffect[id-1] ^= AIO_FIRE
    if( g_nHasEffect[id-1] & AIO_POISON ) g_nHasEffect[id-1] ^= AIO_POISON
    if( g_nHasEffect[id-1] & AIO_NADES )  g_nHasEffect[id-1] ^= AIO_NADES

    return PLUGIN_CONTINUE
}

public Event_ChgWeapon( id )
{
    if( !get_cvar_num( "sv_aio_allow" ) ) return PLUGIN_CONTINUE
    if( !AIO_CommandEnabled( AIO_AMMO ) ) return PLUGIN_CONTINUE
    if ( !(g_nHasEffect[id-1] & AIO_AMMO) ) return PLUGIN_CONTINUE

    new  nClip, nAmmo
    new nWeaponID = get_user_weapon( id, nClip, nAmmo )

    switch( nWeaponID )
    {
        case CSW_C4: return PLUGIN_CONTINUE
        case CSW_HEGRENADE: return PLUGIN_CONTINUE
        case CSW_SMOKEGRENADE: return PLUGIN_CONTINUE
        case CSW_FLASHBANG: return PLUGIN_CONTINUE
        case CSW_KNIFE: return PLUGIN_CONTINUE
    }

    new szWeaponName[STR_T]
    if( nClip == 0 )
    {
        get_weaponname( nWeaponID, szWeaponName, STR_T-1 )
        engclient_cmd( id, "drop", szWeaponName )
        give_item( id, szWeaponName )
        engclient_cmd( id, szWeaponName )
    }
    return PLUGIN_CONTINUE
}

/***************************************************************************
 *******  Client Commands Section  *****************************************
 **************************************************************************/

public ClCmd_AIOMode( id, lvl, cid )
{
    if( !cmd_access( id, lvl, cid, 2 ) )
        return PLUGIN_HANDLED

    new szArg[5]
    read_argv( 1, szArg, 4 )
    new nOnOff = AIO_GetSwitch( szArg )

    set_cvar_num( "sv_aio_allow", nOnOff )
    console_print( id, "[AMXX] AIO-Admin %s!", g_szMode[nOnOff] )

    return PLUGIN_HANDLED
}

public ClCmd_AIOHelp( id, lvl, cid )
{
    if( !cmd_access( id, lvl, cid, 1 ) )
        return PLUGIN_HANDLED

    if( !get_cvar_num( "sv_aio_allow" ) )
    {
        console_print( id, "[AMXX] AIO-Admin Currently DISABLED!" )
        console_print( id, "[AMXX] Use: aio_mode 1 -- To Enable!!!" )
        return PLUGIN_HANDLED
    }

    show_motd( id, g_szFileHelp, "RS AIO-Admin Help!" )

    return PLUGIN_HANDLED
}

public ClCmd_CheckSay( id )
{
    if( g_nHasEffect[id-1] & AIO_GAG )
    {
        client_print( id, print_chat, "[AIO] You Have Been Gagged... You Cannot Chat!" )
        return PLUGIN_HANDLED
    }

    new szSayCmd[STR_T]
    read_argv( 0, szSayCmd, STR_T-1 )

    new i, nNum, nPlayers[MAX_PLAYERS]
    get_players( nPlayers, nNum, "c" )

    for( i = 0; i < nNum; i++ )
    {
        if( g_nHasEffect[nPlayers[i]-1] & AIO_EARS )
        {
            if( (is_user_alive(id) != is_user_alive(nPlayers[i])) || ((get_user_team(id) != get_user_team(nPlayers[i])) && equali(szSayCmd, "say_team")) )
            {
                new szSpeech[STR_M], szPlayer[STR_T]
                read_args( szSpeech, STR_M-1 )
                get_user_name( id, szPlayer, STR_T-1 )
                client_print( nPlayers[i], print_chat, "[BIG-EARS] %s: %s", szPlayer, szSpeech )
            }
        }
    }
    return PLUGIN_CONTINUE
}

public ClCmd_AdminCmd( id, lvl, cid )
{
    if( !cmd_access( id, lvl, cid, 3 ) )
        return PLUGIN_HANDLED

    if( !get_cvar_num( "sv_aio_allow" ) )
    {
        client_print( id, print_console, "[AMXX] AIO-Admin Currently DISABLED" )
        client_print( id, print_chat, "[AMXX] AIO-Admin Currently DISABLED" )
        return PLUGIN_HANDLED
    }

    new szCurrCmd[16]
    new nCmdIdx, nCmdType

    read_argv( 0, szCurrCmd, 15 )
    for( nCmdIdx = 0; nCmdIdx < MAX_COMMANDS; nCmdIdx++ )
        if( equali( szCurrCmd, g_szAdminCmd[nCmdIdx] ) )
            break

    if( !AIO_CommandEnabled( (1<<nCmdIdx) ) )
    {
        console_print( id, "[AMXX] AIO-Admin: %s is DISABLED!", g_szCmdTitle[nCmdIdx] )
        return PLUGIN_HANDLED
    }

    nCmdType = AIO_CommandType( nCmdIdx )

    new szArg[3][STR_T], szTemp[STR_T]
    read_args( szTemp, STR_T-1 )
    ExplodeString( szArg, 3, szTemp, STR_T-1, ' ' )

    new nOnOff = AIO_GetSwitch( szArg[1] )
    copy( g_szCmdArgs[id-1], STR_T-1, szArg[2] )

    if( szArg[0][0] == '@' )
    {
        new nNum = 0, aPlayers[MAX_PLAYERS]
        if( equali(szArg[0][1], "T") || equali(szArg[0][1], "TERRORIST") )
            get_players( aPlayers, nNum, "e", "TERRORIST" )
        else if( equali(szArg[0][1], "C") || equali(szArg[0][1], "CT") )
            get_players( aPlayers, nNum, "e", "CT" )
        else if( equali(szArg[0][1], "A") || equali(szArg[0][1], "ALL") )
            get_players( aPlayers, nNum )

        if( nNum > 0 )
        {
            for( new i = 0; i < nNum; i++ )
            {
                if( AIO_PlayerHasImmunity( aPlayers[i], nCmdType ) )
                {
                    new szAdminName[STR_T]
                    get_user_name( aPlayers[i], szAdminName, STR_T-1 )
                    console_print( id, "[AMXX] AIO_Admin: Player %s Has Immunity", szAdminName )
                }else
                {
                    new aCmdParm[4]
                    aCmdParm[0] = id
                    aCmdParm[1] = aPlayers[i]
                    aCmdParm[2] = nOnOff

                    set_task( 0.5, g_szCmdFuntion[nCmdIdx], (TASK_COMMON + aPlayers[i]), aCmdParm, 3, "a", 0 )
                }
            }
        }else
        {
            console_print( id, "No Clients For %s In Team: %s", g_szCmdTitle[nCmdIdx], szArg[0][1] )
            return  PLUGIN_HANDLED
        }

    }
    else
    {
        new nUserID = cmd_target( id, szArg[0], 7 )
        if( !nUserID )
        {
            console_print( id, "[AMXX] AIO-Admin: User Could Not Be Found For %s...", g_szCmdTitle[nCmdIdx] )
            return PLUGIN_HANDLED
        }

        if( AIO_PlayerHasImmunity( nUserID, nCmdType ) )
        {
            new szAdminName[STR_T]
            get_user_name( nUserID, szAdminName, STR_T-1 )
            console_print( id, "[AMXX] AIO-Admin: Player %s Has Immunity", szAdminName )
        }else
        {
            new aCmdParm[4]
            aCmdParm[0] = id
            aCmdParm[1] = nUserID
            aCmdParm[2] = nOnOff

            set_task( 0.5, g_szCmdFuntion[nCmdIdx], (TASK_COMMON + nUserID), aCmdParm, 3, "a", 0 )
        }
    }
    return PLUGIN_HANDLED
}



/***************************************************************************
 ********  Menu Routines Section  ******************************************
 **************************************************************************/

public ClCmd_AIOMenu( id, lvl, cid )
{
    if( !cmd_access( id, lvl, cid, 1 ) )
        return PLUGIN_HANDLED

    if( !get_cvar_num( "sv_aio_allow" ) )
    {
        client_print( id, print_console, "[AMXX] AIO-Admin Currently DISABLED" )
        client_print( id, print_chat, "[AMXX] AIO-Admin Currently DISABLED" )
        return PLUGIN_HANDLED
    }

    #if defined _DEBUG_MENU
        client_print( id, print_chat, "[DEBUG] ClCmd_AIOMenu Called! (Starts Menu)" )
    #endif

    g_nMenuOrder[id-1] = (get_cvar_num("sv_aio_menu_start")) ? 1 : 0
    g_nCurrCommand[id-1] = AIO_EMPTY
    g_nCurrPlayer[id-1] = AIO_EMPTY
    g_nCurrType[id-1] = AIO_EMPTY

    if( g_nMenuOrder[id-1] )
        Menu_AIOPlayer( id, g_nMenuPosition[id-1] = 0 )
    else
        Menu_AIOCommandType( id )

    return PLUGIN_HANDLED
}

public Menu_AIOPlayer( id, pos )
{
    #if defined _DEBUG_MENU
        client_print( id, print_chat, "[DEBUG] Menu_AIOPlayer Called!" )
        client_print( id, print_chat, "[DEBUG] [Player:%d][Command:%d][Type:%d]", g_nCurrPlayer[id-1], g_nCurrCommand[id-1], g_nCurrType[id-1] )
    #endif

    if( pos < 0 ) return

    g_nCurrPlayer[id-1] = AIO_EMPTY

    get_players( g_nMenuPlayers[id-1], g_nMenuPlayersNum[id-1] )

    new i, j, nCurrKey = 0
    new szUserName[STR_T]
    new szMenuBody[MENU_SIZE]
    new nStart = pos * MENU_DISPLAY

    if( nStart >= g_nMenuPlayersNum[id-1] )
        nStart = pos = g_nMenuPosition[id-1] = 0

    new nLen = format( szMenuBody, MENU_SIZE-1, "\yAIO Player Menu\R%d/%d^n\w^n", (pos+1), (g_nMenuPlayersNum[id-1] / MENU_DISPLAY + ((g_nMenuPlayersNum[id-1] % MENU_DISPLAY) ? 1 : 0 )) )
    new nEnd = nStart + MENU_DISPLAY
    new nKeys = (1<<9)

    if( nEnd > g_nMenuPlayersNum[id-1] )
        nEnd = g_nMenuPlayersNum[id-1]

    for( i = nStart; i < nEnd; i++ )
    {
        j = g_nMenuPlayers[id-1][i]
        get_user_name( j, szUserName, STR_T-1 )

        if( ((g_nCurrType[id-1] != AIO_EMPTY) && AIO_PlayerHasImmunity( j, g_nCurrType[id-1] )) || !is_user_alive(j) )
        {
            nCurrKey++
            nLen += format( szMenuBody[nLen], (MENU_SIZE-1-nLen), "\d%d. %s^n\w", nCurrKey, szUserName )
        }else
        {
            nKeys |= (1<<nCurrKey++)
            nLen += format( szMenuBody[nLen], (MENU_SIZE-1-nLen), "%d. %s^n", nCurrKey, szUserName )
        }
    }

    if( nEnd != g_nMenuPlayersNum[id-1] )
    {
        if( g_nMenuOrder[id-1]  )
            format( szMenuBody[nLen], (MENU_SIZE-1-nLen), "^n9. More...^n0. %s", (pos)?"Back":"Exit" )
        else
            format( szMenuBody[nLen], (MENU_SIZE-1-nLen), "^n9. More...^n0. Back" )
        nKeys |= (1<<8)
    }
    else
    {
        if( g_nMenuOrder[id-1]  )
            format( szMenuBody[nLen], (MENU_SIZE-1-nLen), "^n0. %s", (pos)?"Back":"Exit" )
        else
            format( szMenuBody[nLen], (MENU_SIZE-1-nLen), "^n0. Back" )
    }
    show_menu( id, nKeys, szMenuBody )

    return
}

public Menu_AIOCommandType( id )
{
    #if defined _DEBUG_MENU
        client_print( id, print_chat, "[DEBUG] Menu_AIOCommandType Called!" )
        client_print( id, print_chat, "[DEBUG] [Player:%d][Command:%d][Type:%d]", g_nCurrPlayer[id-1], g_nCurrCommand[id-1], g_nCurrType[id-1] )
    #endif

    g_nCurrCommand[id-1] = AIO_EMPTY
    g_nCurrType[id-1] = AIO_EMPTY
    g_nMenuPosition[id-1] = 0

    new nKeys = (1<<9)
    new szMenuBody[MENU_SIZE]
    new nLen = format( szMenuBody, MENU_SIZE-1, "\yAIO Command Types\w^n" )

    if( (g_nCurrPlayer[id-1] != AIO_EMPTY) )
    {
        if( AIO_PlayerHasImmunity(g_nCurrPlayer[id-1], BAD_CMDS) )
        {
            nLen += format( szMenuBody[nLen], MENU_SIZE-1-nLen, "^n\d1. Evil-Admin Commands" )
        }else
        {
            nKeys |= (1<<0)
            nLen += format( szMenuBody[nLen], MENU_SIZE-1-nLen, "^n\w1. Evil-Admin Commands" )
        }

        if( AIO_PlayerHasImmunity(g_nCurrPlayer[id-1], GOOD_CMDS) )
        {
            nLen += format( szMenuBody[nLen], MENU_SIZE-1-nLen, "^n\d2. Good-Admin Commands" )
        }else
        {
            nKeys |= (1<<1)
            nLen += format( szMenuBody[nLen], MENU_SIZE-1-nLen, "^n\w2. Good-Admin Commands" )
        }

        if( AIO_PlayerHasImmunity(g_nCurrPlayer[id-1], MISC_CMDS) )
        {
            nLen += format( szMenuBody[nLen], MENU_SIZE-1-nLen, "^n\d3. Misc-Admin Commands" )
        }else
        {
            nKeys |= (1<<2)
            nLen += format( szMenuBody[nLen], MENU_SIZE-1-nLen, "^n\w3. Misc-Admin Commands" )
        }
    }else
    {
        nKeys |= ((1<<0)|(1<<1)|(1<<2))
        nLen += format( szMenuBody[nLen], MENU_SIZE-1-nLen, "^n\w1. Evil-Admin Commands" )
        nLen += format( szMenuBody[nLen], MENU_SIZE-1-nLen, "^n\w2. Nice-Admin Commands" )
        nLen += format( szMenuBody[nLen], MENU_SIZE-1-nLen, "^n\w3. Misc-Admin Commands" )
    }
    format( szMenuBody[nLen], MENU_SIZE-1-nLen, "^n^n0. %s", (g_nMenuOrder[id-1])?"Back":"Exit" )

    show_menu( id, nKeys, szMenuBody )

    return
}

public Menu_AIOCommand( id, pos )
{
    #if defined _DEBUG_MENU
        client_print( id, print_chat, "[DEBUG] Menu_AIOCommand Called!" )
        client_print( id, print_chat, "[DEBUG] [Player:%d][Command:%d][Type:%d]", g_nCurrPlayer[id-1], g_nCurrCommand[id-1], g_nCurrType[id-1] )
    #endif

    g_nCurrCommand[id-1] = AIO_EMPTY

    new i, nCurrKey = 0
    new nKeys = (1<<9)
    new nStart, nEnd
    new szMenuBody[MENU_SIZE]
    new nLen = format( szMenuBody, MENU_SIZE-1, "\yAIO Command Menu\R%d/%d^n\w^n", (pos+1), (g_nCurrType[id-1]==MISC_CMDS)?2:1 )

    switch( g_nCurrType[id-1] )
    {
        case BAD_CMDS: nStart = (pos * MENU_DISPLAY)
        case GOOD_CMDS: nStart = ((pos * MENU_DISPLAY) + BAD_CMDS)
        case MISC_CMDS: nStart = ((pos * MENU_DISPLAY) + GOOD_CMDS)
    }
    nEnd = (nStart + MENU_DISPLAY)
    if( nEnd > g_nCurrType[id-1] ) nEnd = g_nCurrType[id-1]

    for( i = nStart; i < nEnd; i++ )
    {
        if( !AIO_CommandEnabled( (1<<i) ) )
        {
            nCurrKey++
            nLen += format( szMenuBody[nLen], (MENU_SIZE-1-nLen), "\d%d. %s^n", nCurrKey, g_szCmdTitle[i] )
        }else
        {
            nKeys |= (1<<nCurrKey++)
            nLen += format( szMenuBody[nLen], (MENU_SIZE-1-nLen), "\w%d. %s^n", nCurrKey, g_szCmdTitle[i] )
        }
    }

    if( nEnd != g_nCurrType[id-1] )
    {
        nKeys |= (1<<8)
        format( szMenuBody[nLen], (MENU_SIZE-1-nLen), "^n\w9. More...^n0. Back" )
    }else
        format( szMenuBody[nLen], (MENU_SIZE-1-nLen), "^n\w0. Back" )

    show_menu( id, nKeys, szMenuBody )

    return
}

public Menu_AIOSettings( id )
{
    #if defined _DEBUG_MENU
        client_print( id, print_chat, "[DEBUG] Menu_AIOSettings Called!" )
        client_print( id, print_chat, "[DEBUG] [Player:%d][Command:%d][Type:%d]", g_nCurrPlayer[id-1], g_nCurrCommand[id-1], g_nCurrType[id-1] )
    #endif

    g_nMenuPosition[id-1] = 0

    new szMenuBody[MENU_SIZE]
    new i, nKeys, nLen = format( szMenuBody, MENU_SIZE-1, "\yAIO Settings Menu\w^n" )

    if( AIO_HasCVAR( (1<<g_nCurrCommand[id-1]) ) )
    {
        nKeys = (1<<9)
        new szSetting[MAX_CMD_CVARS][STR_T]
        new szCvarVal[STR_S]
        new szWeaponName[STR_T], nWeaponID

        get_cvar_string( g_szAdminCVAR[g_nCurrCommand[id-1]], szCvarVal, STR_S-1 )
        ExplodeString( szSetting, MAX_CMD_CVARS, szCvarVal, STR_S-1, ' ' )

        for( i = 0; i < MAX_CMD_CVARS; i++ )
        {
            if( szSetting[i][0] != 0 )
            {
                nKeys |= (1<<i)
                if( (1<<g_nCurrCommand[id-1]) == AIO_WEAPON )
                {
                    nWeaponID = str_to_num(szSetting[i])
                    if( nWeaponID > 0 )
                        get_weaponname( nWeaponID, szWeaponName, STR_T-1 )
                    else
                        copy( szWeaponName, STR_T-1, "None!" )
                    nLen += format( szMenuBody[nLen], (MENU_SIZE-1-nLen), "^n\w%d. %s", (i+1), szWeaponName )
                }else
                {
                    nLen += format( szMenuBody[nLen], (MENU_SIZE-1-nLen), "^n\w%d. %s", (i+1), szSetting[i] )
                }
            }else
                break
        }
        format( szMenuBody[nLen], (MENU_SIZE-1-nLen), "^n^n\w0. Back" )

        show_menu( id, nKeys, szMenuBody )

    }else if( (1<<g_nCurrCommand[id-1]) & AIO_BURY )
    {
        nKeys = ((1<<0)|(1<<1)|(1<<9))
        nLen += format( szMenuBody[nLen], (MENU_SIZE-1-nLen), "^n\w1. Bury" )
        nLen += format( szMenuBody[nLen], (MENU_SIZE-1-nLen), "^n\w2. Unbury" )
        format( szMenuBody[nLen], (MENU_SIZE-1-nLen), "^n^n\w0. Back" )

        show_menu( id, nKeys, szMenuBody )

    }else if( (1<<g_nCurrCommand[id-1]) & AIO_GLOW )
    {
        nKeys = (1<<9)
        for( i = 0; i < MAX_COLORS; i++ )
        {
            nKeys |= (1<<i)
            nLen += format( szMenuBody[nLen], (MENU_SIZE-1-nLen), "^n\w%d. %s", (i+1), g_szColors[i] )
        }
        format( szMenuBody[nLen], (MENU_SIZE-1-nLen), "^n^n\w0. Back" )

        show_menu( id, nKeys, szMenuBody )
    }else
    {
        client_cmd( id, "%s #%d on", g_szAdminCmd[g_nCurrCommand[id-1]], get_user_userid(g_nCurrPlayer[id-1]) )

        if( g_nMenuOrder[id-1] )
            Menu_AIOCommandType( id )
        else
            Menu_AIOPlayer( id, g_nMenuPosition[id-1] = 0 )
    }
    return
}

public Menu_AIONext( id, key )
{
    if( g_nMenuOrder[id-1] && (g_nCurrPlayer[id-1] == AIO_EMPTY) ) // Picked a player 1st
    {
        switch( key )
        {
            case 8: Menu_AIOPlayer( id, ++g_nMenuPosition[id-1] )
            case 9: Menu_AIOPlayer( id, --g_nMenuPosition[id-1] )
            default:
            {
                g_nCurrPlayer[id-1] = g_nMenuPlayers[id-1][g_nMenuPosition[id-1] * MENU_DISPLAY + key]
                Menu_AIOCommandType( id )
            }
        }
    }
    else if( g_nMenuOrder[id-1] && (g_nCurrPlayer[id-1] != AIO_EMPTY) ) // Picked a command 2nd
    {
        switch( key )
        {
            case 8: Menu_AIOCommand( id, ++g_nMenuPosition[id-1] )
            case 9:
            {
                if( g_nMenuPosition[id-1] )
                    Menu_AIOCommand( id, --g_nMenuPosition[id-1] )
                else
                    Menu_AIOCommandType( id )
            }
            default:
            {
                switch( g_nCurrType[id-1] )
                {
                    case BAD_CMDS:  g_nCurrCommand[id-1] = 0
                    case GOOD_CMDS: g_nCurrCommand[id-1] = BAD_CMDS
                    case MISC_CMDS: g_nCurrCommand[id-1] = GOOD_CMDS
                }
                g_nCurrCommand[id-1] += (g_nMenuPosition[id-1] * MENU_DISPLAY + key)
                Menu_AIOSettings( id )
            }
        }
    }
    else if( !g_nMenuOrder[id-1] && (g_nCurrCommand[id-1] == AIO_EMPTY) ) // Picked a command 1st
    {
        switch( key )
        {
            case 8: Menu_AIOCommand( id, ++g_nMenuPosition[id-1] )
            case 9:
            {
                if( g_nMenuPosition[id-1] )
                    Menu_AIOCommand( id, --g_nMenuPosition[id-1] )
                else
                    Menu_AIOCommandType( id )
            }
            default:
            {
                switch( g_nCurrType[id-1] )
                {
                    case BAD_CMDS:  g_nCurrCommand[id-1] = 0
                    case GOOD_CMDS: g_nCurrCommand[id-1] = BAD_CMDS
                    case MISC_CMDS: g_nCurrCommand[id-1] = GOOD_CMDS
                }
                g_nCurrCommand[id-1] += (g_nMenuPosition[id-1] * MENU_DISPLAY + key)
                Menu_AIOPlayer( id, g_nMenuPosition[id-1] = 0 )
            }
        }
    }
    else if( !g_nMenuOrder[id-1] && (g_nCurrCommand[id-1] != AIO_EMPTY) ) // Picked a player 2nd
    {
        switch( key )
        {
            case 8: Menu_AIOPlayer( id, ++g_nMenuPosition[id-1] )
            case 9:
            {
                if( g_nMenuPosition[id-1] )
                    Menu_AIOPlayer( id, --g_nMenuPosition[id-1] )
                else
                    Menu_AIOCommandType( id )
            }
            default:
            {
                g_nCurrPlayer[id-1] = g_nMenuPlayers[id-1][g_nMenuPosition[id-1] * MENU_DISPLAY + key]
                Menu_AIOSettings( id )
            }
        }
    }
    return
}

public Menu_AIOType( id, key )
{
    switch( key )
    {
        case 0: g_nCurrType[id-1] = BAD_CMDS
        case 1: g_nCurrType[id-1] = GOOD_CMDS
        case 2: g_nCurrType[id-1] = MISC_CMDS
        case 9:
        {
            if( g_nMenuOrder[id-1] )
                Menu_AIOPlayer( id, g_nMenuPosition[id-1] = 0 )
            return
        }
    }
    Menu_AIOCommand( id, g_nMenuPosition[id-1] = 0 )
}

public Menu_AIOAction( id, key )
{
    if( key == 9 )
    {
        if( g_nMenuOrder[id-1] )
            Menu_AIOCommandType( id )
        else
            Menu_AIOPlayer( id, g_nMenuPosition[id-1] = 0 )
        return
    }

    if( AIO_HasCVAR( (1<<g_nCurrCommand[id-1]) ) )
    {
        new szSetting[MAX_CMD_CVARS][STR_T], szCvarVal[STR_S]

        get_cvar_string( g_szAdminCVAR[g_nCurrCommand[id-1]], szCvarVal, STR_S-1 )
        ExplodeString( szSetting, MAX_CMD_CVARS, szCvarVal, STR_S-1, ' ' )

        client_cmd( id, "%s #%d on %s", g_szAdminCmd[g_nCurrCommand[id-1]], get_user_userid(g_nCurrPlayer[id-1]), szSetting[key] )

    }else if( (1<<g_nCurrCommand[id-1]) & AIO_BURY )
    {
        client_cmd( id, "%s #%d %d", g_szAdminCmd[g_nCurrCommand[id-1]], get_user_userid(g_nCurrPlayer[id-1]), (key)?0:1 )

    }else if( (1<<g_nCurrCommand[id-1]) & AIO_GLOW )
    {
        client_cmd( id, "%s #%d on %s", g_szAdminCmd[g_nCurrCommand[id-1]], get_user_userid(g_nCurrPlayer[id-1]), g_szColors[key] )
    }

    if( g_nMenuOrder[id-1] )
        Menu_AIOCommandType( id )
    else
        Menu_AIOPlayer( id, g_nMenuPosition[id-1] = 0 )

    return
}


/***************************************************************************
 ********  Action Routines Section (Turns Stuff ON)  ***********************
 **************************************************************************/


public ActionBan( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionBan %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    new szMsg[STR_S], szUserName[STR_T], szUserInfo[STR_T], szCmd[STR_S]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( g_szCmdArgs[p_aCmdArgs[0]-1][0] == 0 )
        copy( g_szCmdArgs[p_aCmdArgs[0]-1], STR_T-1, "0" )

    if( equal( g_szCmdArgs[p_aCmdArgs[0]-1], "0" ) )
    {
        console_print( p_aCmdArgs[1], "You Have Been Banned Permanently!" )
        format( szMsg, STR_S-1, "%s Has Been Banned Permanently!", szUserName )
    }else
    {
        console_print( p_aCmdArgs[1], "You Have Been Banned For %s Minutes!", g_szCmdArgs[p_aCmdArgs[0]-1] )
        format( szMsg, STR_S-1, "%s Has Been Banned For %s Minutes!", szUserName, g_szCmdArgs[p_aCmdArgs[0]-1] )
    }
    AnnounceAction( p_aCmdArgs[0], szMsg )

    if( get_cvar_num( "sv_aio_ban_ip" ) )
    {
        get_user_ip( p_aCmdArgs[1], szUserInfo, STR_T-1, 1 )
        format( szCmd, STR_S-1, "addip ^"%s^" ^"%s^";wait;writeip", g_szCmdArgs[p_aCmdArgs[0]-1], szUserInfo )
    }else
    {
        get_user_authid( p_aCmdArgs[1], szUserInfo, STR_T-1 )
        format( szCmd, STR_S-1, "banid ^"%s^" ^"%s^" kick;wait;writeid", g_szCmdArgs[p_aCmdArgs[0]-1], szUserInfo )
    }

    #if defined _DEBUG_MODE
        client_print( p_aCmdArgs[0], print_chat, szCmd )
    #else
        server_cmd( szCmd )
    #endif

    return
}

public ActionKick( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionKick %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )
    format( szMsg, STR_S-1, "%s Has Just Been Kicked!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )
    console_print( p_aCmdArgs[1], "You Have Been Kicked By Admin!" )

    #if defined _DEBUG_MODE
        client_print( p_aCmdArgs[0], print_chat, "kick #%d", get_user_userid( p_aCmdArgs[1] ) )
    #else
        server_cmd( "kick #%d", get_user_userid( p_aCmdArgs[1] ) )
    #endif

    return
}

public ActionSlap( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionSlap %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( g_szCmdArgs[p_aCmdArgs[0]-1][0] == 0 )
        copy( g_szCmdArgs[p_aCmdArgs[0]-1], STR_T-1, "0" )

    client_print( p_aCmdArgs[1], print_chat, "You Have Been Slapped With %s Damage!", g_szCmdArgs[p_aCmdArgs[0]-1] )
    format( szMsg, STR_S-1, "%s Has Been Slapped With %s Damage!", szUserName, g_szCmdArgs[p_aCmdArgs[0]-1] )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    new aCmdArgs[3]
    aCmdArgs[0] = p_aCmdArgs[1]
    aCmdArgs[1] = str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] )
    BitchSlapPlayer( aCmdArgs )

    return
}

public ActionBitchSlap( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionBitchSlap %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    new nSlapTimes = str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] )
    new aCmdArgs[3]
    aCmdArgs[0] = p_aCmdArgs[1]
    aCmdArgs[1] = 1

    client_print( p_aCmdArgs[1], print_chat, "You Are Being Bitch-Slapped %d Times!", nSlapTimes )
    format( szMsg, STR_S-1, "%s Is Getting Bitch-Slapped %d Times!", szUserName, nSlapTimes )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    if( nSlapTimes < 2 ) nSlapTimes = 2
    set_task( 0.3, "BitchSlapPlayer", (TASK_BSLAP+p_aCmdArgs[1]), aCmdArgs, 2, "a", nSlapTimes-1 )

    return
}

public ActionBury( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        client_print( p_aCmdArgs[0], print_chat, "[DEBUG][AIO] ActionBury %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    client_print( p_aCmdArgs[1], print_chat, "You Have Been %s!", (p_aCmdArgs[2])?"Buried":"Unburied" )
    format( szMsg, STR_S-1, "%s Has Been %s!", szUserName, (p_aCmdArgs[2])?"Buried":"Unburied" )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    new nOrigin[3]
    get_user_origin( p_aCmdArgs[1], nOrigin )
    if( p_aCmdArgs[2] )
        nOrigin[2] -= 30
    else
        nOrigin[2] += 40
    set_user_origin( p_aCmdArgs[1], nOrigin )

    return
}

public ActionGag( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionGag %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    if( p_aCmdArgs[2] )
    {
        if( g_nHasEffect[p_aCmdArgs[1]-1] & AIO_GAG )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Is Currently Gagged!" )
            return
        }
        new Float:fTime = float( str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] ) )
        g_nHasEffect[p_aCmdArgs[1]-1] |= AIO_GAG
        set_task( fTime, "UnGagPlayer", (TASK_GAG + p_aCmdArgs[1]), p_aCmdArgs[1], 1 )
    }else
    {
        if( !(g_nHasEffect[p_aCmdArgs[1]-1] & AIO_GAG) )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Is Not Currently Gagged!" )
            return
        }
        UnGagPlayer( p_aCmdArgs[1] )
        if( task_exists( TASK_GAG + p_aCmdArgs[1] ) )
            remove_task( TASK_GAG + p_aCmdArgs[1] )
    }

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( p_aCmdArgs[2] )
    {
        client_print( p_aCmdArgs[1], print_chat, "You Have Been Gagged For %d Seconds!", str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
        format( szMsg, STR_S-1, "%s Has Been Gagged For %d Seconds!", szUserName, str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
    }else
        format( szMsg, STR_S-1, "%s Has Been Ungagged!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionMute( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionMute %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    if( p_aCmdArgs[2] )
    {
        if( g_nHasEffect[p_aCmdArgs[1]-1] & AIO_MUTE )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Is Currently Muted!" )
            return
        }
        new Float:fTime = float( str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] ) )
        g_nHasEffect[p_aCmdArgs[1]-1] |= AIO_MUTE
        set_speak( p_aCmdArgs[1], SPEAK_MUTED )
        set_task( fTime, "UnMutePlayer", (TASK_MUTE + p_aCmdArgs[1]), p_aCmdArgs[1], 1 )
    }else
    {
        if( !(g_nHasEffect[p_aCmdArgs[1]-1] & AIO_MUTE) )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Is Not Currently Muted!" )
            return
        }
        UnMutePlayer( p_aCmdArgs[1] )
        if( task_exists( TASK_MUTE + p_aCmdArgs[1] ) )
            remove_task( TASK_MUTE + p_aCmdArgs[1] )
    }

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( p_aCmdArgs[2] )
    {
        client_print( p_aCmdArgs[1], print_chat, "You Have Been Muted For %d Seconds!", str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
        format( szMsg, STR_S-1, "%s Has Been Muted For %d Seconds!", szUserName, str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
    }else
        format( szMsg, STR_S-1, "%s Has Been Unmuted!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionWarn( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionWarn %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    client_print( p_aCmdArgs[1], print_chat, "[AIO] You Have Been Warned!" )
    format( szMsg, STR_S-1, "%s Has Been Warned!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    show_motd( p_aCmdArgs[1], g_szFileWarn, "RS AIO-Admin Warning!" )

    return
}

public ActionBlanks( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionBlanks %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    if( p_aCmdArgs[2] )
    {
        if( g_nHasEffect[p_aCmdArgs[1]-1] & AIO_BLANKS )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Is Currently Shooting Blanks!" )
            return
        }
        new Float:fTime = float( str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] ) )
        g_nHasEffect[p_aCmdArgs[1]-1] |= AIO_BLANKS
        set_user_hitzones( p_aCmdArgs[1], 0, 0 )
        set_task( fTime, "LiveAmmoPlayer", (TASK_BLANKS + p_aCmdArgs[1]), p_aCmdArgs[1], 1 )
    }else
    {
        if( !(g_nHasEffect[p_aCmdArgs[1]-1] & AIO_BLANKS) )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Is Currently Shooting Live-Ammo!" )
            return
        }
        LiveAmmoPlayer( p_aCmdArgs[1] )
        if( task_exists( TASK_BLANKS + p_aCmdArgs[1] ) )
            remove_task( TASK_BLANKS + p_aCmdArgs[1] )
    }

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( p_aCmdArgs[2] )
    {
        client_print( p_aCmdArgs[1], print_chat, "You Are Now Shooting Blanks For %d Seconds!", str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
        format( szMsg, STR_S-1, "%s Is Shooting Blanks For %d Seconds!", szUserName, str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
    }else
        format( szMsg, STR_S-1, "%s Is Shooting Live-Ammo!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionFire( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionFire %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    if( p_aCmdArgs[2] )
    {
        if( g_nHasEffect[p_aCmdArgs[1]-1] & AIO_FIRE )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Is Currently Burning!" )
            return
        }
        new Float:fTime = float( str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] ) )
        g_nHasEffect[p_aCmdArgs[1]-1] |= AIO_FIRE
        ignite_effects( p_aCmdArgs[1] )
        ignite_player( p_aCmdArgs[1] )
        set_task( fTime, "ExtinguishPlayer", (TASK_FIRE + p_aCmdArgs[1]), p_aCmdArgs[1], 1 )
    }else
    {
        if( !(g_nHasEffect[p_aCmdArgs[1]-1] & AIO_FIRE) )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Is NOT Currently Burning!" )
            return
        }
        ExtinguishPlayer( p_aCmdArgs[1] )
        if( task_exists( TASK_FIRE + p_aCmdArgs[1] ) )
            remove_task( TASK_FIRE + p_aCmdArgs[1] )
    }

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( p_aCmdArgs[2] )
    {
        client_print( p_aCmdArgs[1], print_chat, "You Are Now On Fire For %d Seconds!", str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
        format( szMsg, STR_S-1, "%s Is On Fire For %d Seconds!", szUserName, str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
    }else
        format( szMsg, STR_S-1, "%s Has Been Extinguished!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionRocket( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionRocket %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    emit_sound( p_aCmdArgs[1], CHAN_WEAPON, g_szSndRocket1, 1.0, ATTN_NORM, 0, PITCH_NORM )
    set_user_maxspeed( p_aCmdArgs[1], 0.01 )
    set_task( 1.2, "RocketPlayer", (TASK_ROCKET+p_aCmdArgs[1]), p_aCmdArgs[1], 2 )

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )
    format( szMsg, STR_S-1, "%s Has Been Turned Into A Rocket!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionStrip( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionStrip %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    StripWeapons( p_aCmdArgs[1] )

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )
    format( szMsg, STR_S-1, "%s Has Been Stripped Of All Guns!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionPoison( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionPoison %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    if( p_aCmdArgs[2] )
    {
        if( g_nHasEffect[p_aCmdArgs[1]-1] & AIO_POISON )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Is Currently Infected!" )
            return
        }
        new Float:fTime = float( str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] ) )
        g_nHasEffect[p_aCmdArgs[1]-1] |= AIO_POISON
        poison_loop( p_aCmdArgs[1] )
        set_task( fTime, "CurePlayer", (TASK_POISON + p_aCmdArgs[1]), p_aCmdArgs[1], 1 )
    }else
    {
        if( !(g_nHasEffect[p_aCmdArgs[1]-1] & AIO_POISON) )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Is NOT Currently Infected!" )
            return
        }
        CurePlayer( p_aCmdArgs[1] )
        if( task_exists( TASK_POISON + p_aCmdArgs[1] ) )
            remove_task( TASK_POISON + p_aCmdArgs[1] )
    }

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( p_aCmdArgs[2] )
    {
        client_print( p_aCmdArgs[1], print_chat, "You Are Now Infected For %d Seconds!", str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
        format( szMsg, STR_S-1, "%s Is Infected For %d Seconds!", szUserName, str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
    }else
        format( szMsg, STR_S-1, "%s Has Been Cured!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionGod( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionGod %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    if( p_aCmdArgs[2] )
    {
        if( g_nHasEffect[p_aCmdArgs[1]-1] & AIO_GOD )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Currently Has God Mode!" )
            return
        }
        new Float:fTime = float( str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] ) )
        g_nHasEffect[p_aCmdArgs[1]-1] |= AIO_GOD
        set_user_godmode( p_aCmdArgs[1], 1 )
        set_task( fTime, "MortalizePlayer", (TASK_GOD + p_aCmdArgs[1]), p_aCmdArgs[1], 1 )
    }else
    {
        if( !(g_nHasEffect[p_aCmdArgs[1]-1] & AIO_GOD) )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Is Currently A Mortal!" )
            return
        }
        MortalizePlayer( p_aCmdArgs[1] )
        if( task_exists( TASK_GOD + p_aCmdArgs[1] ) )
            remove_task( TASK_GOD + p_aCmdArgs[1] )
    }

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( p_aCmdArgs[2] )
    {
        client_print( p_aCmdArgs[1], print_chat, "You Now Have God Mode For %d Seconds!", str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
        format( szMsg, STR_S-1, "%s Has God Mode For %d Seconds!", szUserName, str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
    }else
        format( szMsg, STR_S-1, "%s Is Now A Mortal!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionGrav( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionGrav %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    if( p_aCmdArgs[2] )
    {
        if( g_nHasEffect[p_aCmdArgs[1]-1] & AIO_GRAV )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Currently Has Low Gravity!" )
            return
        }
        new Float:fTime = float( str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] ) )
        g_nHasEffect[p_aCmdArgs[1]-1] |= AIO_GRAV
        set_user_gravity( p_aCmdArgs[1], g_fGravity[1] )
        set_task( fTime, "GravitizePlayer", (TASK_GRAV + p_aCmdArgs[1]), p_aCmdArgs[1], 1 )
    }else
    {
        if( !(g_nHasEffect[p_aCmdArgs[1]-1] & AIO_GRAV) )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Currently Has Normal Gravity!" )
            return
        }
        GravitizePlayer( p_aCmdArgs[1] )
        if( task_exists( TASK_GRAV + p_aCmdArgs[1] ) )
            remove_task( TASK_GRAV + p_aCmdArgs[1] )
    }

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( p_aCmdArgs[2] )
    {
        client_print( p_aCmdArgs[1], print_chat, "You Now Have Low Gravity For %d Seconds!", str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
        format( szMsg, STR_S-1, "%s Has LowGravity For %d Seconds!", szUserName, str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
    }else
        format( szMsg, STR_S-1, "%s Has Normal Gravity!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionNoclip( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionNoclip %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    if( p_aCmdArgs[2] )
    {
        if( g_nHasEffect[p_aCmdArgs[1]-1] & AIO_NOCLIP )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Currently Has No-Clip!" )
            return
        }
        new Float:fTime = float( str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] ) )
        g_nHasEffect[p_aCmdArgs[1]-1] |= AIO_NOCLIP
        set_user_noclip( p_aCmdArgs[1], 1 )
        set_task( fTime, "ClipPlayer", (TASK_NOCLIP + p_aCmdArgs[1]), p_aCmdArgs[1], 1 )
    }else
    {
        if( !(g_nHasEffect[p_aCmdArgs[1]-1] & AIO_NOCLIP) )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Currently Has Normal Clipping!" )
            return
        }
        ClipPlayer( p_aCmdArgs[1] )
        if( task_exists( TASK_NOCLIP + p_aCmdArgs[1] ) )
            remove_task( TASK_NOCLIP + p_aCmdArgs[1] )
    }

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( p_aCmdArgs[2] )
    {
        client_print( p_aCmdArgs[1], print_chat, "You Now Have No-Clip For %d Seconds!", str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
        format( szMsg, STR_S-1, "%s Has No-Clip For %d Seconds!", szUserName, str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
    }else
        format( szMsg, STR_S-1, "%s Has Normal Clipping!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionTurbo( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionTurbo %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    if( p_aCmdArgs[2] )
    {
        if( g_nHasEffect[p_aCmdArgs[1]-1] & AIO_TURBO )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Currently Has Turbo Mode!" )
            return
        }
        new Float:fTime = float( str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] ) )
        new nSpeed = get_cvar_num( "sv_aio_turbo_speed" )

        g_nHasEffect[p_aCmdArgs[1]-1] |= AIO_TURBO
        client_cmd( p_aCmdArgs[1], "cl_forwardspeed %d", nSpeed )
        g_fSpeeds[p_aCmdArgs[1]-1] = get_user_maxspeed( p_aCmdArgs[1] )
        set_user_maxspeed( p_aCmdArgs[1], float(nSpeed) )
        engclient_cmd( p_aCmdArgs[1], "weapon_knife" )
        speed_loop( p_aCmdArgs[1] )
        set_task( fTime, "SlowPlayer", (TASK_TURBO + p_aCmdArgs[1]), p_aCmdArgs[1], 1 )
    }else
    {
        if( !(g_nHasEffect[p_aCmdArgs[1]-1] & AIO_TURBO) )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Currently Has Normal Speed!" )
            return
        }
        SlowPlayer( p_aCmdArgs[1] )
        if( task_exists( TASK_TURBO + p_aCmdArgs[1] ) )
            remove_task( TASK_TURBO + p_aCmdArgs[1] )
    }

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( p_aCmdArgs[2] )
    {
        client_print( p_aCmdArgs[1], print_chat, "You Now Have Turbo Mode For %d Seconds!", str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
        format( szMsg, STR_S-1, "%s Has Turbo Mode For %d Seconds!", szUserName, str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
    }else
        format( szMsg, STR_S-1, "%s Has Normal Speed!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionSilent( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionSilent %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    if( p_aCmdArgs[2] )
    {
        if( g_nHasEffect[p_aCmdArgs[1]-1] & AIO_SILENT )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Currently Has Silent Mode!" )
            return
        }
        new Float:fTime = float( str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] ) )
        g_nHasEffect[p_aCmdArgs[1]-1] |= AIO_SILENT
        set_user_footsteps( p_aCmdArgs[1], 1 )
        set_task( fTime, "UnSilentPlayer", (TASK_SILENT + p_aCmdArgs[1]), p_aCmdArgs[1], 1 )
    }else
    {
        if( !(g_nHasEffect[p_aCmdArgs[1]-1] & AIO_SILENT) )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Currently Makes Noise!" )
            return
        }
        UnSilentPlayer( p_aCmdArgs[1] )
        if( task_exists( TASK_SILENT + p_aCmdArgs[1] ) )
            remove_task( TASK_SILENT + p_aCmdArgs[1] )
    }

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( p_aCmdArgs[2] )
    {
        client_print( p_aCmdArgs[1], print_chat, "You Now Have Silent Mode For %d Seconds!", str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
        format( szMsg, STR_S-1, "%s Has Silent Mode For %d Seconds!", szUserName, str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
    }else
        format( szMsg, STR_S-1, "%s Has Lost Silent Mode!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionInvis( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionInvis %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    if( p_aCmdArgs[2] )
    {
        if( g_nHasEffect[p_aCmdArgs[1]-1] & AIO_INVIS )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Is Currently Invisible!" )
            return
        }
        new Float:fTime = float( str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] ) )
        g_nHasEffect[p_aCmdArgs[1]-1] |= AIO_INVIS
        set_user_rendering( p_aCmdArgs[1], kRenderFxNone, 0,0,0, kRenderTransAdd, 5 )
        set_task( fTime, "VisiblePlayer", (TASK_INVIS + p_aCmdArgs[1]), p_aCmdArgs[1], 1 )
    }else
    {
        if( !(g_nHasEffect[p_aCmdArgs[1]-1] & AIO_INVIS) )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Is Currently Visible!" )
            return
        }
        VisiblePlayer( p_aCmdArgs[1] )
        if( task_exists( TASK_INVIS + p_aCmdArgs[1] ) )
            remove_task( TASK_INVIS + p_aCmdArgs[1] )
    }

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( p_aCmdArgs[2] )
    {
        client_print( p_aCmdArgs[1], print_chat, "You Now Have Invisibility For %d Seconds!", str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
        format( szMsg, STR_S-1, "%s Has Invisibility For %d Seconds!", szUserName, str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
    }else
        format( szMsg, STR_S-1, "%s Has Lost Invisibility!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionSpy( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionSpy %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    if( p_aCmdArgs[2] )
    {
        if( g_nHasEffect[p_aCmdArgs[1]-1] & AIO_SPY )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Currently Looks Like Enemy!" )
            return
        }
        new Float:fTime = float( str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] ) )
        g_nHasEffect[p_aCmdArgs[1]-1] |= AIO_SPY
        new szTeam[16]
        get_user_team( p_aCmdArgs[1], szTeam, 15 )
        if( equali(szTeam, "TERRORIST") )
            cs_set_user_model( p_aCmdArgs[1], g_szCTSkins[random_num(0,3)] )
        else
            cs_set_user_model( p_aCmdArgs[1], g_szTSkins[random_num(0,3)] )
        set_task( fTime, "UnSpyPlayer", (TASK_SPY + p_aCmdArgs[1]), p_aCmdArgs[1], 1 )
    }else
    {
        if( !(g_nHasEffect[p_aCmdArgs[1]-1] & AIO_SPY) )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Currently Looks Normal!" )
            return
        }
        UnSpyPlayer( p_aCmdArgs[1] )
        if( task_exists( TASK_SPY + p_aCmdArgs[1] ) )
            remove_task( TASK_SPY + p_aCmdArgs[1] )
    }

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( p_aCmdArgs[2] )
    {
        client_print( p_aCmdArgs[1], print_chat, "You Now Look Like Enemy For %d Seconds!", str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
        format( szMsg, STR_S-1, "%s Looks Like Enemy For %d Seconds!", szUserName, str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
    }else
        format( szMsg, STR_S-1, "%s Looks Normal!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionNades( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionNades %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    if( p_aCmdArgs[2] )
    {
        if( g_nHasEffect[p_aCmdArgs[1]-1] & AIO_NADES )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Currently Has Unlimited Grenades!" )
            return
        }
        new Float:fTime = float( str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] ) )
        g_nHasEffect[p_aCmdArgs[1]-1] |= AIO_NADES
        grenade_loop( p_aCmdArgs[1] )
        set_task( fTime, "NadelessPlayer", (TASK_NADES + p_aCmdArgs[1]), p_aCmdArgs[1], 1 )
    }else
    {
        if( !(g_nHasEffect[p_aCmdArgs[1]-1] & AIO_NADES) )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Currently Has Limited Grenades!" )
            return
        }
        NadelessPlayer( p_aCmdArgs[1] )
        if( task_exists( TASK_NADES + p_aCmdArgs[1] ) )
            remove_task( TASK_NADES + p_aCmdArgs[1] )
    }

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( p_aCmdArgs[2] )
    {
        client_print( p_aCmdArgs[1], print_chat, "You Have Unlimited Grenades For %d Seconds!", str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
        format( szMsg, STR_S-1, "%s Has Unlimited Grenades For %d Seconds!", szUserName, str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
    }else
        format( szMsg, STR_S-1, "%s Has Limited Grenades!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionAmmo( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionAmmo %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    if( p_aCmdArgs[2] )
    {
        if( g_nHasEffect[p_aCmdArgs[1]-1] & AIO_AMMO )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Currently Has Unlimited Ammo!" )
            return
        }
        new Float:fTime = float( str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] ) )
        g_nHasEffect[p_aCmdArgs[1]-1] |= AIO_AMMO
        set_task( fTime, "AmmolessPlayer", (TASK_AMMO + p_aCmdArgs[1]), p_aCmdArgs[1], 1 )
    }else
    {
        if( !(g_nHasEffect[p_aCmdArgs[1]-1] & AIO_AMMO) )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Currently Has Limited Ammo!" )
            return
        }
        AmmolessPlayer( p_aCmdArgs[1] )
        if( task_exists( TASK_AMMO + p_aCmdArgs[1] ) )
            remove_task( TASK_AMMO + p_aCmdArgs[1] )
    }

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( p_aCmdArgs[2] )
    {
        client_print( p_aCmdArgs[1], print_chat, "You Have Unlimited Ammo For %d Seconds!", str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
        format( szMsg, STR_S-1, "%s Has Unlimited Ammo For %d Seconds!", szUserName, str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
    }else
        format( szMsg, STR_S-1, "%s Has Limited Ammo!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionHealth( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionHealth %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( g_szCmdArgs[p_aCmdArgs[0]-1][0] == 0 )
        copy( g_szCmdArgs[p_aCmdArgs[0]-1], STR_T-1, "100" )

    client_print( p_aCmdArgs[1], print_chat, "Your HP Has Been Set To: %s!", g_szCmdArgs[p_aCmdArgs[0]-1] )
    format( szMsg, STR_S-1, "%s Has Been Set With %s HP!", szUserName, g_szCmdArgs[p_aCmdArgs[0]-1] )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    set_user_health( p_aCmdArgs[1], str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] ) )

    return
}

public ActionArmor( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionArmor %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( g_szCmdArgs[p_aCmdArgs[0]-1][0] == 0 )
        copy( g_szCmdArgs[p_aCmdArgs[0]-1], STR_T-1, "100" )

    client_print( p_aCmdArgs[1], print_chat, "Your AP Has Been Set To: %s!", g_szCmdArgs[p_aCmdArgs[0]-1] )
    format( szMsg, STR_S-1, "%s Has Been Set With %s AP!", szUserName, g_szCmdArgs[p_aCmdArgs[0]-1] )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    set_user_armor( p_aCmdArgs[1], str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] ) )

    return
}

public ActionRevival( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionRevival %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    if( p_aCmdArgs[2] )
    {
        if( g_nHasEffect[p_aCmdArgs[1]-1] & AIO_REVIVE )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Currently Has Instant Revival!" )
            return
        }
        g_nHasEffect[p_aCmdArgs[1]-1] |= AIO_REVIVE
        g_nReviveTimes[p_aCmdArgs[1]-1] = str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] )
    }else
    {
        if( !(g_nHasEffect[p_aCmdArgs[1]-1] & AIO_REVIVE) )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Does Not Have Instant Revival!" )
            return
        }
        DeadPlayer( p_aCmdArgs[1] )
    }

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( p_aCmdArgs[2] )
    {
        client_print( p_aCmdArgs[1], print_chat, "You Have Instant Revival For %d Deaths!", str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
        format( szMsg, STR_S-1, "%s Has Instant Revival For %d Deaths!", szUserName, str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
    }else
        format( szMsg, STR_S-1, "%s Will Stay Dead Now!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionEars( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionEars %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    if( p_aCmdArgs[2] )
    {
        if( g_nHasEffect[p_aCmdArgs[1]-1] & AIO_EARS )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Currently Has Big Ears!" )
            return
        }
        new Float:fTime = float( str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] ) )
        g_nHasEffect[p_aCmdArgs[1]-1] |= AIO_EARS
        set_speak( p_aCmdArgs[1], SPEAK_LISTENALL )
        set_task( fTime, "DeafenPlayer", (TASK_EARS + p_aCmdArgs[1]), p_aCmdArgs[1], 1 )
    }else
    {
        if( !(g_nHasEffect[p_aCmdArgs[1]-1] & AIO_EARS) )
        {
            console_print( p_aCmdArgs[0], "[AIO] Client Currently Has Normal Ears!" )
            return
        }
        DeafenPlayer( p_aCmdArgs[1] )
        if( task_exists( TASK_EARS + p_aCmdArgs[1] ) )
            remove_task( TASK_EARS + p_aCmdArgs[1] )
    }

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( p_aCmdArgs[2] )
    {
        client_print( p_aCmdArgs[1], print_chat, "You Now Have Big Ears For %d Seconds!", str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
        format( szMsg, STR_S-1, "%s Has Big Ears For %d Seconds!", szUserName, str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]) )
    }else
        format( szMsg, STR_S-1, "%s Has Lost Big Ears!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionStack( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionStack %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    client_print( 0, print_chat, "You Have Been Stacked!" )
    format( szMsg, STR_S-1, "Everyone Has Been Stacked on %s!", szUserName )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    new nOrigin[3], i, nNum
    new nPlayers[MAX_PLAYERS]
    new nOffsetY = 36, nOffsetZ = 96

    get_user_origin( p_aCmdArgs[1], nOrigin )
    get_players( nPlayers, nNum, "a" )

    if( p_aCmdArgs[2] )
        nOffsetZ = 0
    else nOffsetY = 0

    for( i = 0; i < nNum; i++ )
    {
        if( (nPlayers[i] == p_aCmdArgs[1]) || AIO_PlayerHasImmunity( nPlayers[i], MISC_CMDS ) )
            continue

        nOrigin[1] += nOffsetY
        nOrigin[2] += nOffsetZ

        set_user_origin( nPlayers[i], nOrigin )
    }
    return
}

public ActionGlow( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionGlow %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    client_print( 0, print_chat, "You Now Have A %s Glow!", g_szCmdArgs[p_aCmdArgs[0]-1] )
    format( szMsg, STR_S-1, "%s Has A %s Glow!", szUserName, g_szCmdArgs[p_aCmdArgs[0]-1] )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    new nColor
    for( nColor = 0; nColor < MAX_COLORS; nColor++ )
        if( equali(g_szCmdArgs[p_aCmdArgs[0]-1], g_szColors[nColor]) )
            break

    if( nColor == MAX_COLORS-1 ) nColor = random_num( 0, MAX_COLORS-2 )
    set_user_rendering( p_aCmdArgs[1], kRenderFxGlowShell, g_nColors[nColor][0], g_nColors[nColor][1], g_nColors[nColor][2], kRenderNormal, 30 )

    return
}

public ActionCash( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionCash %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( g_szCmdArgs[p_aCmdArgs[0]-1][0] == 0 )
        copy( g_szCmdArgs[p_aCmdArgs[0]-1], STR_T-1, "16000" )

    new nCash = clamp( str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1]), 0, 16000 )
    cs_set_user_money( p_aCmdArgs[1], nCash )

    client_print( p_aCmdArgs[1], print_chat, "Your Cash Has Been Set To: %d!", nCash )
    format( szMsg, STR_S-1, "%s Has Been Set With %d Cash!", szUserName, nCash )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionAdmin( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionAdmin %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( p_aCmdArgs[2] )
    {
        if( g_szCmdArgs[p_aCmdArgs[0]-1][0] == 0 )
            copy( g_szCmdArgs[p_aCmdArgs[0]-1], STR_T-1, "i" )

        client_print( p_aCmdArgs[1], print_chat, "Your Priveleges Have Been Set To: %s!", g_szCmdArgs[p_aCmdArgs[0]-1] )
        format( szMsg, STR_S-1, "%s Has Been Set With %s Priveleges!", szUserName, g_szCmdArgs[p_aCmdArgs[0]-1] )
        set_user_flags( p_aCmdArgs[1], read_flags(g_szCmdArgs[p_aCmdArgs[0]-1]) )
    }else
    {
        client_print( p_aCmdArgs[1], print_chat, "Your Priveleges Have Been Removed!", g_szCmdArgs[p_aCmdArgs[0]-1] )
        format( szMsg, STR_S-1, "%s Had Priveleges Removed!", szUserName, g_szCmdArgs[p_aCmdArgs[0]-1] )
        set_user_flags( p_aCmdArgs[1], (-1) )
    }
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionFrags( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionFrags %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( g_szCmdArgs[p_aCmdArgs[0]-1][0] == 0 )
        copy( g_szCmdArgs[p_aCmdArgs[0]-1], STR_T-1, "1" )

    new nFrags = (get_user_frags( p_aCmdArgs[1] ) + str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] ))
    set_user_frags( p_aCmdArgs[1], nFrags )

    client_print( p_aCmdArgs[1], print_chat, "Your Frags Have Been Set To: %d!", nFrags )
    format( szMsg, STR_S-1, "%s Has Been Set With %d Frags!", szUserName, nFrags )
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}

public ActionWeapon( p_aCmdArgs[] )
{
    #if defined _DEBUG_MODE
        console_print( p_aCmdArgs[0], "[DEBUG][AIO] ActionWeapon %d %d %s", p_aCmdArgs[1], p_aCmdArgs[2], g_szCmdArgs[p_aCmdArgs[0]-1] )
    #endif

    new szMsg[STR_S], szUserName[STR_T]
    get_user_name( p_aCmdArgs[1], szUserName, STR_T-1 )

    if( g_szCmdArgs[p_aCmdArgs[0]-1][0] == 0 )
        copy( g_szCmdArgs[p_aCmdArgs[0]-1], STR_T-1, "26" )  // Default DEAGLE

    if( str_to_num(g_szCmdArgs[p_aCmdArgs[0]-1][0]) == 0 )
    {
        client_print( p_aCmdArgs[1], print_chat, "You Have Been Striped of All Weapons!" )
        format( szMsg, STR_S-1, "%s Has Been Striped of All Weapons!", szUserName )
        StripWeapons( p_aCmdArgs[1] )
    }else
    {
        new szWeaponName[STR_T]
        get_weaponname( str_to_num( g_szCmdArgs[p_aCmdArgs[0]-1] ), szWeaponName, STR_T-1 )
        give_item( p_aCmdArgs[1], szWeaponName )
        client_print( p_aCmdArgs[1], print_chat, "You Have Been Given %s!", szWeaponName[7] )
        format( szMsg, STR_S-1, "%s Has Been Given %s!", szUserName, szWeaponName[7] )
    }
    AnnounceAction( p_aCmdArgs[0], szMsg )

    return
}


/***************************************************************************
 ********  Action Routines Section (Turns Stuff OFF)  **********************
 **************************************************************************/

public BitchSlapPlayer( p_aCmdArgs[] )
    user_slap( p_aCmdArgs[0], p_aCmdArgs[1] )

public UnGagPlayer( p_aCmdArgs[] )
{
    client_print( p_aCmdArgs[0], print_chat, "You Have Been Ungagged And May Now Chat!" )
    g_nHasEffect[p_aCmdArgs[0]-1] ^= AIO_GAG
}

public UnMutePlayer( p_aCmdArgs[] )
{
    client_print( p_aCmdArgs[0], print_chat, "You Have Been Unmuted And May Now Speak!" )
    g_nHasEffect[p_aCmdArgs[0]-1] ^= AIO_MUTE
    set_speak( p_aCmdArgs[0], SPEAK_NORMAL )
}

public LiveAmmoPlayer( p_aCmdArgs[] )
{
    client_print( p_aCmdArgs[0], print_chat, "You Are Now Shooting Live-Ammo!" )
    g_nHasEffect[p_aCmdArgs[0]-1] ^= AIO_BLANKS
    set_user_hitzones( p_aCmdArgs[1], 0, 255 )
}

public ExtinguishPlayer( p_aCmdArgs[] )
{
    client_print( p_aCmdArgs[0], print_chat, "You Have Been Extinguished!" )
    if( g_nHasEffect[p_aCmdArgs[0]-1] & AIO_FIRE )
        g_nHasEffect[p_aCmdArgs[0]-1] ^= AIO_FIRE
}

public RocketPlayer( p_aCmdArgs[] )
{
    client_print( p_aCmdArgs[0], print_chat, "You Are Now An Explosive Rocket!" )
    set_user_gravity( p_aCmdArgs[0], -0.50 )
    client_cmd( p_aCmdArgs[0], "+jump;wait;wait;-jump" )
    emit_sound( p_aCmdArgs[0], CHAN_VOICE, g_szSndRocket2, 1.0, 0.5, 0, PITCH_NORM )
    rocket_effects( p_aCmdArgs[0] )
}

public CurePlayer( p_aCmdArgs[] )
{
    client_print( p_aCmdArgs[0], print_chat, "You Have Been Cured Of Your Infection!" )
    if( g_nHasEffect[p_aCmdArgs[0]-1] & AIO_POISON )
        g_nHasEffect[p_aCmdArgs[0]-1] ^= AIO_POISON
}

public MortalizePlayer( p_aCmdArgs[] )
{
    client_print( p_aCmdArgs[0], print_chat, "You Have Been Returned To Mortal State!" )
    g_nHasEffect[p_aCmdArgs[0]-1] ^= AIO_GOD
    set_user_godmode( p_aCmdArgs[0], 0 )
}

public GravitizePlayer( p_aCmdArgs[] )
{
    client_print( p_aCmdArgs[0], print_chat, "You Have Normal Gravity Now!" )
    g_nHasEffect[p_aCmdArgs[0]-1] ^= AIO_GRAV
    set_user_gravity( p_aCmdArgs[0], g_fGravity[0] )
}

public ClipPlayer( p_aCmdArgs[] )
{
    client_print( p_aCmdArgs[0], print_chat, "You Have Normal Clipping Now!" )
    g_nHasEffect[p_aCmdArgs[0]-1] ^= AIO_NOCLIP
    set_user_noclip( p_aCmdArgs[0], 0 )
}

public SlowPlayer( p_aCmdArgs[] )
{
    client_print( p_aCmdArgs[0], print_chat, "You Have Lost Your Turbo Speed!" )
    g_nHasEffect[p_aCmdArgs[0]-1] ^= AIO_TURBO
}

public UnSilentPlayer( p_aCmdArgs[] )
{
    client_print( p_aCmdArgs[0], print_chat, "You Have Lost Your Silent Mode!" )
    g_nHasEffect[p_aCmdArgs[0]-1] ^= AIO_SILENT
    set_user_footsteps( p_aCmdArgs[0], 0 )
}

public VisiblePlayer( p_aCmdArgs[] )
{
    client_print( p_aCmdArgs[0], print_chat, "You Have Lost Invisibility!" )
    g_nHasEffect[p_aCmdArgs[0]-1] ^= AIO_INVIS
    set_user_rendering( p_aCmdArgs[0], kRenderFxNone, 255,255,255, kRenderNormal, 255 )
}

public UnSpyPlayer( p_aCmdArgs[] )
{
    client_print( p_aCmdArgs[0], print_chat, "You Now Look Normal!" )
    g_nHasEffect[p_aCmdArgs[0]-1] ^= AIO_SPY
    cs_reset_user_model( p_aCmdArgs[0] )
}

public NadelessPlayer( p_aCmdArgs[] )
{
    client_print( p_aCmdArgs[0], print_chat, "You Have Limited Grenades Again!" )
    if( g_nHasEffect[p_aCmdArgs[0]-1] & AIO_NADES )
        g_nHasEffect[p_aCmdArgs[0]-1] ^= AIO_NADES
}

public AmmolessPlayer( p_aCmdArgs[] )
{
    client_print( p_aCmdArgs[0], print_chat, "You Have Limited Ammo Again!" )
    if( g_nHasEffect[p_aCmdArgs[0]-1] & AIO_AMMO )
        g_nHasEffect[p_aCmdArgs[0]-1] ^= AIO_AMMO
}

public DeadPlayer( p_aCmdArgs[] )
{
    client_print( p_aCmdArgs[0], print_chat, "You Will Stay Dead Now!" )
    if( g_nHasEffect[p_aCmdArgs[0]-1] & AIO_REVIVE )
        g_nHasEffect[p_aCmdArgs[0]-1] ^= AIO_REVIVE
    g_nReviveTimes[p_aCmdArgs[0]-1] = 0
}

public DeafenPlayer( p_aCmdArgs[] )
{
    client_print( p_aCmdArgs[0], print_chat, "You Have Lost Big Ears!" )
    g_nHasEffect[p_aCmdArgs[0]-1] ^= AIO_EARS
    set_speak( p_aCmdArgs[0], SPEAK_NORMAL )
}

/***************************************************************************
 ********  Respawn Control for Revival  ************************************
 **************************************************************************/

public RevivePlayer( p_aCmdArgs[] )
{
    if( get_user_team( p_aCmdArgs[0] ) == 3 || is_user_alive( p_aCmdArgs[0] ) )
        return PLUGIN_CONTINUE

    spawn( p_aCmdArgs[0] )

    return PLUGIN_CONTINUE
}

/***************************************************************************
 ********  Speed Loop  *****************************************************
 **************************************************************************/

public speed_loop( p_aCmdArgs[] )
{
    if( g_nHasEffect[p_aCmdArgs[0]-1] & AIO_TURBO )
    {
        new nSpeed = get_cvar_num( "sv_aio_turbo_speed" )
        set_user_maxspeed( p_aCmdArgs[0], float(nSpeed) )
        set_task( 1.0, "speed_loop", 0, p_aCmdArgs, 2 )
    }else
    {
        client_cmd( p_aCmdArgs[0], "cl_forwardspeed 400" )
        set_user_maxspeed( p_aCmdArgs[0], g_fSpeeds[p_aCmdArgs[0]-1] )
        g_fSpeeds[p_aCmdArgs[0]-1] = 0.0
        engclient_cmd( p_aCmdArgs[0], "weapon_knife" )
    }
    return
}

/***************************************************************************
 ********  Unlimited Grenade Loop  *****************************************
 **************************************************************************/

public grenade_loop( p_aCmdArgs[] )
{
    new vID = p_aCmdArgs[0]
    if( is_user_alive(vID) && (g_nHasEffect[vID-1] & AIO_NADES) )
    {
        new i, nNum, nWeapon[32]
        new bool:hasNade = false
        get_user_weapons( vID, nWeapon, nNum )

        for( i = 0; i < nNum; i++ )
        {
            if( nWeapon[i] == CSW_HEGRENADE )
            {
                hasNade = true
                break
            }
        }
        if( !hasNade ) give_item( vID, "weapon_hegrenade" )

        set_task( 1.0, "grenade_loop", 0, p_aCmdArgs, 2 )
    }
    else
    {
        if( g_nHasEffect[vID-1] & AIO_NADES )
            NadelessPlayer( p_aCmdArgs )
    }
    return PLUGIN_CONTINUE
}

/***************************************************************************
 ********  Fire Effects By f117bomb (Slightly Modified)  *******************
 **************************************************************************/

public ignite_effects( skIndex[] )
{
    new kIndex = skIndex[0]

    if( is_user_alive(kIndex) && (g_nHasEffect[kIndex-1] & AIO_FIRE) )
    {
        new korigin[3]
        get_user_origin( kIndex, korigin )

        //TE_SPRITE - additive sprite, plays 1 cycle
        message_begin( MSG_BROADCAST, SVC_TEMPENTITY )
        write_byte( 17 )
        write_coord( korigin[0] )  // coord, coord, coord (position)
        write_coord( korigin[1] )
        write_coord( korigin[2] )
        write_short( g_sprMFlash ) // short (sprite index)
        write_byte( 20 )           // byte (scale in 0.1's)
        write_byte( 200 )          // byte (brightness)
        message_end()

        //Smoke
        message_begin( MSG_BROADCAST, SVC_TEMPENTITY, korigin )
        write_byte( 5 )
        write_coord( korigin[0] )  // coord coord coord (position)
        write_coord( korigin[1] )
        write_coord( korigin[2] )
        write_short( g_sprSmoke )  // short (sprite index)
        write_byte( 20 )           // byte (scale in 0.1's)
        write_byte( 15 )           // byte (framerate)
        message_end()

        set_task( 0.2, "ignite_effects", 0, skIndex, 2 )
    }
    else
    {
        if( g_nHasEffect[kIndex-1] & AIO_FIRE )
        {
            emit_sound( kIndex, CHAN_AUTO, g_szSndScream2, 0.6, ATTN_NORM, 0, PITCH_HIGH )
            g_nHasEffect[kIndex-1] ^= AIO_FIRE
        }
    }
    return PLUGIN_CONTINUE
}

public ignite_player( skIndex[] )
{
    new kIndex = skIndex[0]

    if( is_user_alive(kIndex) && (g_nHasEffect[kIndex-1] & AIO_FIRE) )
    {
        new korigin[3]
        new players[32], inum = 0
        new pOrigin[3]
        new kHealth = get_user_health( kIndex )
        new nDmgAmount = get_cvar_num( "sv_aio_fire_damage" )
        get_user_origin( kIndex, korigin )

        //create some damage
        set_user_health( kIndex, (kHealth - nDmgAmount) )

        message_begin( MSG_ONE, g_nMsgDamage, {0,0,0}, kIndex )
        write_byte( 30 )          // dmg_save
        write_byte( 30 )          // dmg_take
        write_long( 1<<21 )       // visibleDamageBits
        write_coord( korigin[0] ) // damageOrigin.x
        write_coord( korigin[1] ) // damageOrigin.y
        write_coord( korigin[2] ) // damageOrigin.z
        message_end()

        //create some sound
        emit_sound( kIndex, CHAN_ITEM, g_szSndFlames, 0.6, ATTN_NORM, 0, PITCH_NORM )

        //Ignite Others
        get_players( players, inum, "a" )
        for( new i = 0; i < inum; ++i )
        {
            get_user_origin( players[i], pOrigin )
            if( get_distance( korigin, pOrigin ) < 100  )
            {
                if( !(g_nHasEffect[players[i]-1] & AIO_FIRE) )
                {
                    new spIndex[2]
                    spIndex[0] = players[i]
                    new pName[32], kName[32]
                    get_user_name( players[i], pName, 31 )
                    get_user_name( kIndex, kName, 31 )
                    emit_sound( players[i], CHAN_WEAPON, g_szSndScream1, 1.0, ATTN_NORM, 0, PITCH_HIGH )
                    client_print( 0, print_chat, "[AIO] OH NO! %s has caught %s on fire!", kName, pName )
                    g_nHasEffect[players[i]-1] |= AIO_FIRE
                    ignite_player( players[i] )
                    ignite_effects( players[i] )
                }
            }
        }
        players[0] = 0
        pOrigin[0] = 0
        korigin[0] = 0

        //Call Again in 2 seconds
        set_task( 2.0, "ignite_player", 0, skIndex, 2 )
    }

    return PLUGIN_CONTINUE
}

/***************************************************************************
 ********  Rocket Effects By f117bomb (Slightly Modified)  *****************
 **************************************************************************/

public rocket_effects( svictim[] )
{
    new victim = svictim[0]

    if( is_user_alive(victim) )
    {
        new vorigin[3]
        get_user_origin( victim, vorigin )

        message_begin( MSG_ONE, g_nMsgDamage, {0,0,0}, victim )
        write_byte( 30 )          // dmg_save
        write_byte( 30 )          // dmg_take
        write_long( 1<<16 )       // visibleDamageBits
        write_coord( vorigin[0] ) // damageOrigin.x
        write_coord( vorigin[1] ) // damageOrigin.y
        write_coord( vorigin[2] ) // damageOrigin.z
        message_end()

        if( g_nRocket_Z[victim-1] == vorigin[2] )
            rocket_explode( svictim )
        g_nRocket_Z[victim-1] = vorigin[2]

        //Draw Trail and effects
        //TE_SPRITETRAIL - line of moving glow sprites with gravity, fadeout, and collisions
        message_begin( MSG_BROADCAST, SVC_TEMPENTITY )
        write_byte( 15 )
        write_coord( vorigin[0] )        // coord, coord, coord (start)
        write_coord( vorigin[1] )
        write_coord( vorigin[2] )
        write_coord( vorigin[0] )        // coord, coord, coord (end)
        write_coord( vorigin[1] )
        write_coord( vorigin[2] - 30 )
        write_short( g_sprBFlare )       // short (sprite index)
        write_byte( 5 )                  // byte (count)
        write_byte( 1 )                  // byte (life in 0.1's)
        write_byte( 1 )                  // byte (scale in 0.1's)
        write_byte( 10 )                 // byte (velocity along vector in 10's)
        write_byte( 5 )                  // byte (randomness of velocity in 10's)
        message_end()

        //TE_SPRITE - additive sprite, plays 1 cycle
        message_begin( MSG_BROADCAST,SVC_TEMPENTITY)
        write_byte( 17 )
        write_coord( vorigin[0] )        // coord, coord, coord (position)
        write_coord( vorigin[1] )
        write_coord( vorigin[2] - 30 )
        write_short( g_sprMFlash )       // short (sprite index)
        write_byte( 15 )                 // byte (scale in 0.1's)
        write_byte( 255 )                // byte (brightness)
        message_end()

        set_task( 0.2, "rocket_effects", 0, svictim, 2 )
    }
    return PLUGIN_CONTINUE
}

public rocket_explode( svictim[] )
{
    new victim = svictim[0]

    if( is_user_alive(victim) ) /*If user is alive create effects and user_kill */
    {
        new vec1[3]
        get_user_origin( victim, vec1 )

        // blast circles
        message_begin( MSG_BROADCAST, SVC_TEMPENTITY, vec1 )
        write_byte( 21 )
        write_coord( vec1[0] )
        write_coord( vec1[1] )
        write_coord( vec1[2] - 10 )
        write_coord( vec1[0] )
        write_coord( vec1[1] )
        write_coord( vec1[2] + 1910 )
        write_short( g_sprWhite )
        write_byte( 0 )              // startframe
        write_byte( 0 )              // framerate
        write_byte( 2 )              // life
        write_byte( 16 )             // width
        write_byte( 0 )              // noise
        write_byte( 188 )            // r
        write_byte( 220 )            // g
        write_byte( 255 )            // b
        write_byte( 255 )            //brightness
        write_byte( 0 )              // speed
        message_end()

        //Explosion2
        message_begin( MSG_BROADCAST, SVC_TEMPENTITY )
        write_byte( 12 )
        write_coord( vec1[0] )
        write_coord( vec1[1] )
        write_coord( vec1[2] )
        write_byte( 188 )            // byte (scale in 0.1's)
        write_byte( 10 )             // byte (framerate)
        message_end()

        //Smoke
        message_begin( MSG_BROADCAST,SVC_TEMPENTITY,vec1)
        write_byte( 5 )
        write_coord(vec1[0])
        write_coord(vec1[1])
        write_coord(vec1[2])
        write_short( g_sprSmoke )
        write_byte( 2 )
        write_byte( 10 )
        message_end()

        user_kill( victim, 1 )
    }

    //stop_sound
    emit_sound( victim, CHAN_VOICE, g_szSndRocket2, 0.0, 0.0, (1<<5), PITCH_NORM )

    set_user_maxspeed( victim, 1.0 )
    set_user_gravity( victim, 1.00 )

    return PLUGIN_CONTINUE
}

/***************************************************************************
 ********  Poison Effects By AssKicR (Slightly Modified)  ******************
 **************************************************************************/

public poison_loop( p_aCmdArgs[] )
{
    new vID = p_aCmdArgs[0]
    if( is_user_alive(vID) && (g_nHasEffect[vID-1] & AIO_POISON) )
    {
        new userHealth = get_user_health( vID )
        new nDmgAmount = get_cvar_num( "sv_aio_poison_damage" )
        new nNewHP = (userHealth - nDmgAmount)
        set_user_health( vID, nNewHP )
        if( nNewHP < 1 )
        {
            message_begin( MSG_BROADCAST, g_nMsgDeath, {0,0,0}, 0 )
            write_byte( vID )
            write_byte( vID )
            write_byte( 0 )
            write_string( "infection" )
            message_end()
        }

        new korigin[3]
        get_user_origin( vID, korigin )
        message_begin( MSG_ONE, g_nMsgDamage, {0,0,0}, vID )
        write_byte( 30 )             // dmg_save
        write_byte( 30 )             // dmg_take
        write_long( 1 << 16 )        // visibleDamageBits
        write_coord( korigin[0] )    // damageOrigin.x
        write_coord( korigin[1] )    // damageOrigin.y
        write_coord( korigin[2] )    // damageOrigin.z
        message_end()

        emit_sound( vID, CHAN_AUTO, g_szSndPoison, 0.6, ATTN_NORM, 0, PITCH_HIGH )

        set_task( 1.0, "poison_loop", 0, p_aCmdArgs, 2 )
    }
    else
    {
        if( g_nHasEffect[vID-1] & AIO_POISON )
        {
            emit_sound( vID, CHAN_AUTO, g_szSndScream2, 0.6, ATTN_NORM, 0, PITCH_HIGH )
            g_nHasEffect[vID-1] ^= AIO_POISON
        }
    }
    return PLUGIN_CONTINUE
}